import { jsx } from 'react/jsx-runtime';
import vtk from '@kitware/vtk.js/vtk';
import { useCallback, useEffect, useMemo } from 'react';
import deletionRegistry from '../utils/DeletionRegistry.js';
import useGetterRef from '../utils/useGetterRef.js';
import { usePrevious } from '../utils/usePrevious.js';
import useUnmount from '../utils/useUnmount.js';
import { useRepresentation, useDownstream, DownstreamContext } from './contexts.js';

function Algorithm(props) {
    const prev = usePrevious(props);
    const { vtkClass, state, port = 0 } = props;
    const createAlgo = useCallback(() => {
        if (typeof vtkClass === 'string') {
            return vtk(Object.assign({ vtkClass }, state));
        }
        return vtkClass.newInstance(state);
    }, [vtkClass, state]);
    const [algoRef, getAlgorithm] = useGetterRef(() => {
        const algo = createAlgo();
        deletionRegistry.register(algo, () => algo.delete());
        return algo;
    });
    const representation = useRepresentation();
    const downstream = useDownstream();
    useEffect(() => {
        let algoChanged = false;
        if (!algoRef.current) {
            algoRef.current = createAlgo();
            algoChanged = true;
        }
        else if (vtkClass !== (prev === null || prev === void 0 ? void 0 : prev.vtkClass)) {
            const curAlgo = algoRef.current;
            const newAlgo = createAlgo();
            // copy connections over from current algorithm
            const nbInputs = curAlgo.getNumberOfInputPorts();
            for (let i = 0; i < nbInputs; i++) {
                const connection = curAlgo.getInputConnection(i);
                if (connection) {
                    newAlgo.setInputConnection(connection, i);
                }
                else {
                    newAlgo.setInputData(curAlgo.getInputData(i), i);
                }
            }
            algoRef.current = newAlgo;
            deletionRegistry.markForDeletion(curAlgo);
            algoChanged = true;
        }
        else if (state) {
            const modified = algoRef.current.set(state);
            if (modified) {
                representation.dataChanged();
            }
        }
        if (algoChanged) {
            downstream.setInputConnection(algoRef.current.getOutputPort(), port);
            // if a source algo, then it already produces data
            if (algoRef.current.getNumberOfInputPorts() === 0) {
                representation.dataAvailable();
            }
        }
    }, [
        prev,
        vtkClass,
        state,
        port,
        createAlgo,
        algoRef,
        representation,
        downstream,
    ]);
    useUnmount(() => {
        if (algoRef.current) {
            deletionRegistry.markForDeletion(algoRef.current);
            algoRef.current = null;
        }
    });
    const fwdDownstream = useMemo(() => ({
        setInputData: (...args) => getAlgorithm().setInputData(...args),
        setInputConnection: (...args) => getAlgorithm().setInputConnection(...args),
    }), [getAlgorithm]);
    return (jsx(DownstreamContext.Provider, Object.assign({ value: fwdDownstream }, { children: props.children })));
}

export { Algorithm as default };
