import vtkDataArray from '@kitware/vtk.js/Common/Core/DataArray';
import { TYPED_ARRAYS } from '@kitware/vtk.js/macros';
import { useEffect } from 'react';
import { TypedArrayLookup, toTypedArray } from '../utils/index.js';
import deletionRegistry from '../utils/DeletionRegistry.js';
import useGetterRef from '../utils/useGetterRef.js';
import { usePrevious } from '../utils/usePrevious.js';
import useUnmount from '../utils/useUnmount.js';
import { useFieldData, useDataset } from './contexts.js';

const DefaultProps = {
    registration: 'addArray',
    name: 'scalars',
    type: 'Float32Array',
    numberOfComponents: 1,
};
function DataArray(props) {
    const prev = usePrevious(Object.assign(Object.assign({}, DefaultProps), props));
    const [daRef, getDataArray] = useGetterRef(() => {
        const da = vtkDataArray.newInstance({
            name: 'scalars',
            empty: true,
        });
        deletionRegistry.register(da, () => da.delete());
        return da;
    });
    const getFieldData = useFieldData();
    const dataset = useDataset();
    const { registration = DefaultProps.registration } = props;
    const { name = DefaultProps.name, type = DefaultProps.type, values = [], numberOfComponents = DefaultProps.numberOfComponents, range, } = props;
    // register array with the dataset must happen before dataset.modified()
    // cleanup should happend before the polydata is deleted
    useEffect(() => {
        const array = getDataArray();
        const ds = dataset.getDataSet();
        deletionRegistry.incRefCount(ds);
        const fieldData = getFieldData();
        const register = fieldData[registration];
        register(array);
        ds.modified();
        return () => {
            fieldData.removeArray(array.getName());
            deletionRegistry.decRefCount(ds);
        };
    }, [registration, dataset, getDataArray, getFieldData]);
    useEffect(() => {
        const array = getDataArray();
        let modified = !prev;
        modified = array.setName(name) || modified;
        modified = type !== (prev === null || prev === void 0 ? void 0 : prev.type) || modified;
        modified = numberOfComponents !== (prev === null || prev === void 0 ? void 0 : prev.numberOfComponents) || modified;
        modified = values !== (prev === null || prev === void 0 ? void 0 : prev.values) || modified;
        if (modified) {
            const typedArrayClass = TypedArrayLookup[TYPED_ARRAYS[type]];
            array.setData(toTypedArray(values, typedArrayClass), numberOfComponents);
            if (range) {
                array.setRange(range, numberOfComponents);
            }
        }
        if (modified)
            dataset.modified();
    });
    useUnmount(() => {
        if (daRef.current) {
            deletionRegistry.markForDeletion(daRef.current);
            daRef.current = null;
        }
    });
    return null;
}

export { DataArray as default };
