import { jsx } from 'react/jsx-runtime';
import vtkActor from '@kitware/vtk.js/Rendering/Core/Actor';
import vtkMapper from '@kitware/vtk.js/Rendering/Core/Mapper';
import { forwardRef, useState, useEffect, useMemo, useImperativeHandle } from 'react';
import { compareShallowObject } from '../utils/comparators.js';
import useBooleanAccumulator from '../utils/useBooleanAccumulator.js';
import useComparableEffect from '../utils/useComparableEffect.js';
import { useRendererContext, RepresentationContext, DownstreamContext } from './contexts.js';
import useColorTransferFunction from './modules/useColorTransferFunction.js';
import useMapper from './modules/useMapper.js';
import useProp from './modules/useProp.js';

const DefaultProps = {
    colorMapPreset: 'erdc_rainbow_bright',
    colorDataRange: [0, 1],
};
var GeometryRepresentation = forwardRef(function GeometryRepresentation(props, fwdRef) {
    var _a, _b, _c, _d;
    const [modifiedRef, trackModified, resetModified] = useBooleanAccumulator();
    const [dataAvailable, setDataAvailable] = useState(false);
    // --- LUT --- //
    const getLookupTable = useColorTransferFunction((_a = props.colorMapPreset) !== null && _a !== void 0 ? _a : DefaultProps.colorMapPreset, (_b = props.colorDataRange) !== null && _b !== void 0 ? _b : DefaultProps.colorDataRange, trackModified);
    // --- mapper --- //
    const getMapper = useMapper(() => vtkMapper.newInstance({
        lookupTable: getLookupTable(),
        useLookupTableScalarRange: true,
    }), props.mapper, trackModified);
    useEffect(() => {
        getMapper().setLookupTable(getLookupTable());
    }, [getMapper, getLookupTable]);
    // --- actor --- //
    const actorProps = Object.assign(Object.assign({}, props.actor), { visibility: dataAvailable && ((_d = (_c = props.actor) === null || _c === void 0 ? void 0 : _c.visibility) !== null && _d !== void 0 ? _d : true) });
    const getActor = useProp({
        constructor: () => vtkActor.newInstance({ visibility: false }),
        id: props.id,
        props: actorProps,
        trackModified,
    });
    useEffect(() => {
        getActor().setMapper(getMapper());
    }, [getActor, getMapper]);
    // set actor property props
    const { property: propertyProps } = props;
    useComparableEffect(() => {
        if (!propertyProps)
            return;
        trackModified(getActor().getProperty().set(propertyProps));
    }, [propertyProps], ([cur], [prev]) => compareShallowObject(cur, prev));
    // --- //
    const renderer = useRendererContext();
    useEffect(() => {
        if (modifiedRef.current) {
            renderer.requestRender();
            resetModified();
        }
    });
    const representation = useMemo(() => ({
        dataChanged: () => {
            renderer.requestRender();
        },
        dataAvailable: () => {
            setDataAvailable(true);
            representation.dataChanged();
        },
        getActor,
        getMapper,
    }), [renderer, getActor, getMapper]);
    const downstream = useMemo(() => ({
        setInputData: (...args) => getMapper().setInputData(...args),
        setInputConnection: (...args) => getMapper().setInputConnection(...args),
    }), [getMapper]);
    useImperativeHandle(fwdRef, () => representation);
    return (jsx(RepresentationContext.Provider, Object.assign({ value: representation }, { children: jsx(DownstreamContext.Provider, Object.assign({ value: downstream }, { children: props.children })) })));
});

export { GeometryRepresentation as default };
