import { jsx } from 'react/jsx-runtime';
import vtkPolyData from '@kitware/vtk.js/Common/DataModel/PolyData';
import { forwardRef, useMemo, useImperativeHandle, useEffect } from 'react';
import { toTypedArray, dataArraySize } from '../utils/index.js';
import deletionRegistry from '../utils/DeletionRegistry.js';
import useGetterRef from '../utils/useGetterRef.js';
import { usePrevious } from '../utils/usePrevious.js';
import useUnmount from '../utils/useUnmount.js';
import { useRepresentation, useDownstream, DatasetContext } from './contexts.js';

const DefaultProps = {
    port: 0,
    points: [],
    connectivity: 'manual',
};
var PolyData = forwardRef(function PolyData(props, fwdRef) {
    const { port = DefaultProps.port, points = DefaultProps.points, connectivity = DefaultProps.connectivity, verts, lines, polys, strips, } = props;
    const [pdRef, getPolyData] = useGetterRef(() => {
        const pd = vtkPolyData.newInstance();
        deletionRegistry.register(pd, () => pd.delete());
        return pd;
    });
    const representation = useRepresentation();
    const downstream = useDownstream();
    // dataset API
    const dataset = useMemo(() => ({
        getDataSet: getPolyData,
        modified: () => {
            const pd = getPolyData();
            pd.modified();
            downstream.setInputData(pd, port);
            // Let the representation know that we have data
            if (pd.getPoints().getData()) {
                representation.dataAvailable();
            }
        },
    }), [port, representation, downstream, getPolyData]);
    useImperativeHandle(fwdRef, () => dataset);
    const prev = usePrevious(Object.assign(Object.assign({}, DefaultProps), props));
    // update polydata
    useEffect(() => {
        const polyData = getPolyData();
        let modified = !prev;
        let typedArrayClass = Uint32Array;
        // TODO shallow array equals? Or tell developer to do this?
        if (points && points !== (prev === null || prev === void 0 ? void 0 : prev.points)) {
            modified = true;
            const array = toTypedArray(points, Float64Array);
            polyData.getPoints().setData(array, 3);
            // Adapt cell size
            // Max cell size for uint16 is 655356*3=196608.
            // switch to uint32array if this is the case.
            typedArrayClass = array.length > 196608 ? Uint32Array : Uint16Array;
        }
        if (verts && verts !== (prev === null || prev === void 0 ? void 0 : prev.verts)) {
            modified = true;
            polyData.getVerts().setData(toTypedArray(verts, typedArrayClass));
        }
        if (lines && lines !== (prev === null || prev === void 0 ? void 0 : prev.lines)) {
            modified = true;
            polyData.getLines().setData(toTypedArray(lines, typedArrayClass));
        }
        if (polys && polys !== (prev === null || prev === void 0 ? void 0 : prev.polys)) {
            modified = true;
            polyData.getPolys().setData(toTypedArray(polys, typedArrayClass));
        }
        if (strips && strips !== (prev === null || prev === void 0 ? void 0 : prev.strips)) {
            modified = true;
            polyData.getStrips().setData(toTypedArray(strips, typedArrayClass));
        }
        const curSize = dataArraySize(points);
        const prevSize = (prev === null || prev === void 0 ? void 0 : prev.points) ? dataArraySize(prev.points) : NaN;
        if (connectivity !== (prev === null || prev === void 0 ? void 0 : prev.connectivity) || curSize !== prevSize) {
            modified = true;
            const nbPoints = curSize / 3;
            switch (connectivity) {
                case 'points':
                    {
                        const values = new Uint32Array(nbPoints + 1);
                        values[0] = nbPoints;
                        for (let i = 0; i < nbPoints; i++) {
                            values[i + 1] = i;
                        }
                        polyData.getVerts().setData(values);
                    }
                    break;
                case 'triangles':
                    {
                        const values = new Uint32Array(nbPoints + nbPoints / 3);
                        let offset = 0;
                        for (let i = 0; i < nbPoints; i += 3) {
                            values[offset++] = 3;
                            values[offset++] = i + 0;
                            values[offset++] = i + 1;
                            values[offset++] = i + 2;
                        }
                        polyData.getPolys().setData(values);
                    }
                    break;
                case 'strips':
                    {
                        const values = new Uint32Array(nbPoints + 1);
                        values[0] = nbPoints;
                        for (let i = 0; i < nbPoints; i++) {
                            values[i + 1] = i;
                        }
                        polyData.getStrips().setData(values);
                    }
                    break;
                // do nothing for manual or anything else...
            }
        }
        if (modified) {
            polyData.computeBounds();
            dataset.modified();
        }
    });
    useUnmount(() => {
        if (pdRef.current) {
            deletionRegistry.markForDeletion(pdRef.current);
            pdRef.current = null;
        }
    });
    return (jsx(DatasetContext.Provider, Object.assign({ value: dataset }, { children: props.children })));
});

export { PolyData as default };
