import { toArrayBuffer } from '@kitware/vtk.js/Common/Core/Base64.js';
import vtk from '@kitware/vtk.js/vtk';
import { useCallback, useRef, useEffect } from 'react';
import deletionRegistry from '../utils/DeletionRegistry.js';
import { useRepresentation, useDownstream } from './contexts.js';

/**
 * Reads data and produces vtkObjects.
 */
function Reader(props) {
    const { vtkClass, url, urlOptions, arrayBuffer, base64ArrayBuffer, text } = props;
    if ([url, arrayBuffer, base64ArrayBuffer, text].filter(Boolean).length > 1) {
        console.warn('<Reader>: multiple data options provided. Please only provide one.');
    }
    const representation = useRepresentation();
    const createReader = useCallback(() => {
        if (typeof vtkClass === 'string') {
            return vtk({ vtkClass });
        }
        return vtkClass.newInstance();
    }, [vtkClass]);
    // --- reader creation --- //
    const readerRef = useRef(null);
    useEffect(() => {
        const reader = createReader();
        deletionRegistry.register(reader, () => reader.delete());
        readerRef.current = reader;
        return () => {
            deletionRegistry.markForDeletion(reader);
        };
    }, [createReader]);
    // --- url handling --- //
    useEffect(() => {
        const reader = readerRef.current;
        if (!reader || !url)
            return;
        reader.setUrl(url, urlOptions).then(() => {
            if (reader.isDeleted())
                return;
            representation.dataAvailable();
        });
    }, [url, urlOptions, representation]);
    // --- arrayBuffer --- //
    useEffect(() => {
        const reader = readerRef.current;
        if (!reader || !arrayBuffer)
            return;
        reader.parseAsArrayBuffer(arrayBuffer);
        representation.dataAvailable();
    }, [arrayBuffer, representation]);
    // --- base64 string of ArrayBuffer --- //
    useEffect(() => {
        const reader = readerRef.current;
        if (!reader || !base64ArrayBuffer)
            return;
        reader.parseAsArrayBuffer(toArrayBuffer(base64ArrayBuffer));
        representation.dataAvailable();
    }, [base64ArrayBuffer, representation]);
    // --- text --- //
    useEffect(() => {
        const reader = readerRef.current;
        if (!reader || !text)
            return;
        reader.parseAsText(text);
        representation.dataAvailable();
    }, [text, representation]);
    // --- downstream registration --- //
    const downstream = useDownstream();
    const { port } = props;
    useEffect(() => {
        const reader = readerRef.current;
        if (!reader)
            return;
        downstream.setInputConnection(reader.getOutputPort(), port);
    });
}

export { Reader as default };
