import { jsx } from 'react/jsx-runtime';
import vtkRenderWindow from '@kitware/vtk.js/Rendering/Core/RenderWindow';
import { forwardRef, useContext, useEffect, useRef, useCallback, useMemo, useImperativeHandle } from 'react';
import deletionRegistry from '../utils/DeletionRegistry.js';
import useGetterRef from '../utils/useGetterRef.js';
import useMount from '../utils/useMount.js';
import useResizeObserver from '../utils/useResizeObserver.js';
import useUnmount from '../utils/useUnmount.js';
import { OpenGLRenderWindowContext, RenderWindowContext } from './contexts.js';
import useInteractor from './modules/useInteractor.js';

var RenderWindow = forwardRef(function RenderWindow(props, fwdRef) {
    const openGLRenderWindow = useContext(OpenGLRenderWindowContext);
    if (!openGLRenderWindow)
        throw new Error('No OpenGL Render Window!');
    // --- RenderWindow --- //
    const [rwRef, getRenderWindow] = useGetterRef(() => {
        const rw = vtkRenderWindow.newInstance();
        deletionRegistry.register(rw, () => rw.delete());
        return rw;
    });
    useEffect(() => {
        const rwView = openGLRenderWindow.get();
        deletionRegistry.incRefCount(rwView);
        const renderWindow = getRenderWindow();
        renderWindow.addView(rwView);
        return () => {
            renderWindow.removeView(rwView);
            deletionRegistry.decRefCount(rwView);
        };
    }, [openGLRenderWindow, getRenderWindow]);
    useUnmount(() => {
        if (rwRef.current) {
            deletionRegistry.markForDeletion(rwRef.current);
            rwRef.current = null;
        }
    });
    // --- Interactor --- //
    const getInteractor = useInteractor(openGLRenderWindow);
    useEffect(() => {
        getRenderWindow().setInteractor(getInteractor());
    });
    // --- rendering --- //
    const renderTimeoutRef = useRef(null);
    const queueRender = useCallback(() => {
        if (renderTimeoutRef.current == null) {
            renderTimeoutRef.current = setTimeout(() => {
                renderTimeoutRef.current = null;
                const renderWindow = getRenderWindow();
                renderWindow.render();
            });
        }
    }, [getRenderWindow]);
    // --- resize --- //
    const updateViewSize = useCallback(() => {
        const container = openGLRenderWindow.getContainer();
        if (!container)
            return;
        const renderWindowView = openGLRenderWindow.get();
        const devicePixelRatio = window.devicePixelRatio || 1;
        const { width, height } = container.getBoundingClientRect();
        const w = Math.floor(width * devicePixelRatio);
        const h = Math.floor(height * devicePixelRatio);
        renderWindowView.setSize(Math.max(w, 10), Math.max(h, 10));
        queueRender();
    }, [openGLRenderWindow, queueRender]);
    useResizeObserver(openGLRenderWindow.get().getContainer(), updateViewSize);
    useMount(() => updateViewSize());
    // --- api --- //
    const api = useMemo(() => ({
        get: getRenderWindow,
        getInteractor,
        requestRender: queueRender,
    }), [getRenderWindow, getInteractor, queueRender]);
    useImperativeHandle(fwdRef, () => api);
    return (jsx(RenderWindowContext.Provider, Object.assign({ value: api }, { children: props.children })));
});

export { RenderWindow as default };
