import { jsx } from 'react/jsx-runtime';
import { useRef, useCallback, useMemo, useEffect } from 'react';
import useUnmount from '../utils/useUnmount.js';
import { ShareDataSetContext, useShareDataSet, RepresentationContext, DownstreamContext, useRepresentation, useDownstream } from './contexts.js';

const DATA_AVAILABLE_EVENT = 'dataAvailable';
const DATA_CHANGED_EVENT = 'dataChanged';
function ShareDataSetRoot(props) {
    const datasets = useRef(new Map());
    const eventTarget = useRef(new EventTarget());
    // Used to notify UseDataSet components that register
    // after the DATA_AVAILABLE_EVENT occurs.
    const dataAvailable = useRef(new Set());
    useUnmount(() => {
        datasets.current.clear();
        dataAvailable.current.clear();
    });
    const createDataEventRegistrar = useCallback((event, options) => {
        return function addListener(name, callback) {
            const invoke = () => {
                var _a;
                callback((_a = datasets.current.get(name)) !== null && _a !== void 0 ? _a : null);
            };
            const handler = (ev) => {
                var _a;
                if (!(ev instanceof (CustomEvent)))
                    return;
                if (name === ((_a = ev.detail) === null || _a === void 0 ? void 0 : _a.name)) {
                    invoke();
                }
            };
            let immediate = false;
            if (typeof (options === null || options === void 0 ? void 0 : options.immediate) === 'boolean') {
                immediate = options.immediate;
            }
            else if ((options === null || options === void 0 ? void 0 : options.immediate) instanceof Function) {
                immediate = options === null || options === void 0 ? void 0 : options.immediate(name);
            }
            if (immediate) {
                invoke();
            }
            eventTarget.current.addEventListener(event, handler);
            return () => {
                eventTarget.current.removeEventListener(event, handler);
            };
        };
    }, []);
    const share = useMemo(() => ({
        register(name, dataset) {
            datasets.current.set(name, dataset);
        },
        unregister(name) {
            datasets.current.delete(name);
            dataAvailable.current.delete(name);
        },
        dispatchDataAvailable(name) {
            dataAvailable.current.add(name);
            eventTarget.current.dispatchEvent(new CustomEvent(DATA_AVAILABLE_EVENT, {
                detail: { name },
            }));
        },
        dispatchDataChanged(name) {
            eventTarget.current.dispatchEvent(new CustomEvent(DATA_CHANGED_EVENT, {
                detail: { name },
            }));
        },
        onDataChanged: createDataEventRegistrar(DATA_CHANGED_EVENT),
        onDataAvailable: createDataEventRegistrar(DATA_AVAILABLE_EVENT, {
            immediate(name) {
                return dataAvailable.current.has(name);
            },
        }),
    }), [createDataEventRegistrar]);
    return (jsx(ShareDataSetContext.Provider, Object.assign({ value: share }, { children: props.children })));
}
function RegisterDataSet(props) {
    const share = useShareDataSet();
    const { id } = props;
    // --- handle registrations --- //
    useEffect(() => () => {
        share.unregister(id);
    }, [id, share]);
    useUnmount(() => {
        share.unregister(id);
    });
    // --- //
    const downstream = useMemo(() => ({
        setInputData(obj) {
            share.register(id, obj);
        },
        setInputConnection(conn) {
            downstream.setInputData(conn());
        },
    }), [id, share]);
    const mockRepresentation = useMemo(() => ({
        dataChanged() {
            share.dispatchDataChanged(id);
        },
        dataAvailable() {
            share.dispatchDataAvailable(id);
        },
        getActor: () => null,
        getMapper: () => null,
    }), [id, share]);
    return (jsx(RepresentationContext.Provider, Object.assign({ value: mockRepresentation }, { children: jsx(DownstreamContext.Provider, Object.assign({ value: downstream }, { children: props.children })) })));
}
function UseDataSet(props) {
    const { id, port = 0 } = props;
    const share = useShareDataSet();
    // TODO if useDataSet is input to an algorithm, should representation be null?
    const representation = useRepresentation();
    const downstream = useDownstream();
    useEffect(() => {
        return share.onDataAvailable(id, (ds) => {
            downstream.setInputData(ds, port);
            representation.dataAvailable();
        });
    }, [id, port, representation, downstream, share]);
    useEffect(() => {
        return share.onDataChanged(id, () => {
            representation.dataChanged();
        });
    }, [id, representation, share]);
    return null;
}

export { RegisterDataSet, ShareDataSetRoot, UseDataSet };
