import { jsx } from 'react/jsx-runtime';
import vtkImageMapper from '@kitware/vtk.js/Rendering/Core/ImageMapper';
import vtkImageSlice from '@kitware/vtk.js/Rendering/Core/ImageSlice';
import { forwardRef, useState, useEffect, useMemo, useImperativeHandle } from 'react';
import { compareShallowObject } from '../utils/comparators.js';
import useBooleanAccumulator from '../utils/useBooleanAccumulator.js';
import useComparableEffect from '../utils/useComparableEffect.js';
import { useRendererContext, RepresentationContext, DownstreamContext } from './contexts.js';
import useColorTransferFunction from './modules/useColorTransferFunction.js';
import useMapper from './modules/useMapper.js';
import useProp from './modules/useProp.js';

const DefaultProps = {
    colorMapPreset: 'Grayscale',
    colorDataRange: 'auto',
};
var SliceRepresentation = forwardRef(function SliceRepresentation(props, fwdRef) {
    var _a, _b, _c, _d;
    const [modifiedRef, trackModified, resetModified] = useBooleanAccumulator();
    const [dataAvailable, setDataAvailable] = useState(false);
    const rangeFromProps = (_a = props.colorDataRange) !== null && _a !== void 0 ? _a : DefaultProps.colorDataRange;
    const colorDataRange = rangeFromProps === 'auto' ? [0, 1] : rangeFromProps;
    // --- LUT --- //
    const getLookupTable = useColorTransferFunction((_b = props.colorMapPreset) !== null && _b !== void 0 ? _b : DefaultProps.colorMapPreset, colorDataRange, trackModified);
    // --- PWF --- //
    // --- mapper --- //
    const getMapper = useMapper(() => vtkImageMapper.newInstance(), props.mapper, trackModified);
    // --- actor --- //
    const actorProps = Object.assign(Object.assign({}, props.actor), { visibility: dataAvailable && ((_d = (_c = props.actor) === null || _c === void 0 ? void 0 : _c.visibility) !== null && _d !== void 0 ? _d : true) });
    const getActor = useProp({
        constructor: () => vtkImageSlice.newInstance(),
        id: props.id,
        props: actorProps,
        trackModified,
    });
    useEffect(() => {
        getActor().setMapper(getMapper());
    }, [getActor, getMapper]);
    useEffect(() => {
        getActor().getProperty().setRGBTransferFunction(0, getLookupTable());
        getActor().getProperty().setInterpolationTypeToLinear();
    }, [getActor, getLookupTable]);
    // set actor property props
    const { property: propertyProps } = props;
    useComparableEffect(() => {
        if (!propertyProps)
            return;
        trackModified(getActor().getProperty().set(propertyProps));
    }, [propertyProps], ([cur], [prev]) => compareShallowObject(cur, prev));
    // --- Slice changes --- //
    const { iSlice, jSlice, kSlice, xSlice, ySlice, zSlice } = props;
    useEffect(() => {
        if (iSlice != null)
            trackModified(getMapper().setISlice(iSlice));
    }, [iSlice, getMapper, trackModified]);
    useEffect(() => {
        if (jSlice != null)
            trackModified(getMapper().setJSlice(jSlice));
    }, [jSlice, getMapper, trackModified]);
    useEffect(() => {
        if (kSlice != null)
            trackModified(getMapper().setKSlice(kSlice));
    }, [kSlice, getMapper, trackModified]);
    useEffect(() => {
        if (xSlice != null)
            trackModified(getMapper().setXSlice(xSlice));
    }, [xSlice, getMapper, trackModified]);
    useEffect(() => {
        if (ySlice != null)
            trackModified(getMapper().setYSlice(ySlice));
    }, [ySlice, getMapper, trackModified]);
    useEffect(() => {
        if (zSlice != null)
            trackModified(getMapper().setZSlice(zSlice));
    }, [zSlice, getMapper, trackModified]);
    // --- //
    const renderer = useRendererContext();
    useEffect(() => {
        if (modifiedRef.current) {
            renderer.requestRender();
            resetModified();
        }
    });
    const representation = useMemo(() => ({
        dataChanged: () => {
            renderer.requestRender();
        },
        dataAvailable: () => {
            setDataAvailable(true);
            representation.dataChanged();
        },
        getActor,
        getMapper,
    }), [renderer, getActor, getMapper]);
    const downstream = useMemo(() => ({
        setInputData: (...args) => getMapper().setInputData(...args),
        setInputConnection: (...args) => getMapper().setInputConnection(...args),
    }), [getMapper]);
    useImperativeHandle(fwdRef, () => representation);
    return (jsx(RepresentationContext.Provider, Object.assign({ value: representation }, { children: jsx(DownstreamContext.Provider, Object.assign({ value: downstream }, { children: props.children })) })));
});

export { SliceRepresentation as default };
