import { jsx } from 'react/jsx-runtime';
import { forwardRef, useRef, useContext, useCallback, useEffect, useMemo, useImperativeHandle } from 'react';
import { pick } from '../../utils/index.js';
import { ResizeWatcherContext } from '../../utils/ResizeWatcher.js';
import { useEventListener } from '../../utils/useEventListener.js';
import useMount from '../../utils/useMount.js';
import { OpenGLRenderWindowContext, RenderWindowContext, ViewContext } from '../contexts.js';
import useApplyCenterOfRotation from '../modules/useApplyCenterOfRotation.js';
import { useInteractorStyle, useInteractorStyleManipulatorSettings } from '../modules/useInteractorStyle.js';
import Renderer from '../Renderer.js';
import { DefaultProps } from './view-shared.js';

/**
 * A View that is within a MultiViewRoot.
 */
const ParentedView = forwardRef(function ParentedView(props, fwdRef) {
    const containerRef = useRef(null);
    const rendererRef = useRef(null);
    const rendererProps = pick(props, 'background', 'interactive', 'camera', 'autoResetCamera');
    const openGLRenderWindowAPI = useContext(OpenGLRenderWindowContext);
    if (!openGLRenderWindowAPI) {
        throw new Error('OpenGLRenderWindow is missing inside the MultiViewRoot');
    }
    const renderWindowAPI = useContext(RenderWindowContext);
    if (!renderWindowAPI) {
        throw new Error('RenderWindow is missing inside the MultiViewRoot');
    }
    // --- interactor & style binding --- //
    const getInteractor = useCallback(() => renderWindowAPI.getInteractor(), [renderWindowAPI]);
    const [getInteractorStyle, setInteractorStyle] = useInteractorStyle(getInteractor);
    const { interactorSettings = DefaultProps.interactorSettings, autoCenterOfRotation = DefaultProps.autoCenterOfRotation, } = props;
    useInteractorStyleManipulatorSettings(getInteractorStyle, interactorSettings);
    useApplyCenterOfRotation(rendererRef, getInteractorStyle, autoCenterOfRotation);
    /**
     * 1. Switch to targeted renderer.
     * 2. Switch to this View's interactor style.
     */
    useEventListener(containerRef, 'pointerenter', (ev) => {
        const rendererAPI = rendererRef.current;
        if (!rendererAPI)
            return;
        const interactor = getInteractor();
        interactor.setCurrentRenderer(rendererAPI.get());
        interactor.setInteractorStyle(getInteractorStyle());
        const oldContainer = interactor.getContainer();
        const newContainer = containerRef.current;
        if (oldContainer !== newContainer) {
            if (oldContainer) {
                interactor.unbindEvents();
            }
            if (newContainer) {
                interactor.bindEvents(newContainer);
            }
            // forward event to interactor
            interactor.handlePointerEnter(ev);
        }
    });
    // --- resize handling --- //
    const resizeWatcher = useContext(ResizeWatcherContext);
    const onResize = useCallback(() => {
        const renderer = rendererRef.current;
        const viewContainer = openGLRenderWindowAPI.getContainer();
        const rendererContainer = containerRef.current;
        if (!renderer || !viewContainer || !rendererContainer)
            return;
        const viewBox = viewContainer.getBoundingClientRect();
        const rendererBox = rendererContainer.getBoundingClientRect();
        const top = rendererBox.top - viewBox.top;
        const left = rendererBox.left - viewBox.left;
        const xmin = left / viewBox.width;
        const xmax = (left + rendererBox.width) / viewBox.width;
        const ymin = 1 - (top + rendererBox.height) / viewBox.height;
        const ymax = 1 - top / viewBox.height;
        renderer.get().setViewport(xmin, ymin, xmax, ymax);
        renderer.requestRender();
    }, [openGLRenderWindowAPI]);
    useMount(() => onResize());
    useEffect(() => {
        const rendererContainer = containerRef.current;
        const renderer = rendererRef.current;
        const viewContainer = openGLRenderWindowAPI.getContainer();
        if (resizeWatcher && rendererContainer && renderer && viewContainer) {
            resizeWatcher.watch(rendererContainer, onResize);
            resizeWatcher.watch(viewContainer, onResize);
            return () => {
                resizeWatcher.unwatch(rendererContainer, onResize);
                resizeWatcher.unwatch(viewContainer, onResize);
            };
        }
    }, [onResize, openGLRenderWindowAPI, resizeWatcher]);
    // --- api --- //
    const api = useMemo(() => ({
        isInMultiViewRoot: () => true,
        getViewContainer: () => containerRef.current,
        getOpenGLRenderWindow: () => openGLRenderWindowAPI,
        getRenderWindow: () => renderWindowAPI,
        getRenderer: () => rendererRef.current,
        getInteractorStyle: () => getInteractorStyle(),
        setInteractorStyle: (style) => setInteractorStyle(style),
        requestRender: () => { var _a; return (_a = rendererRef.current) === null || _a === void 0 ? void 0 : _a.requestRender(); },
        getCamera: () => { var _a, _b; return (_b = (_a = rendererRef.current) === null || _a === void 0 ? void 0 : _a.get().getActiveCamera()) !== null && _b !== void 0 ? _b : null; },
        resetCamera: (boundsToUse) => { var _a; return (_a = rendererRef.current) === null || _a === void 0 ? void 0 : _a.resetCamera(boundsToUse); },
    }), [
        openGLRenderWindowAPI,
        renderWindowAPI,
        getInteractorStyle,
        setInteractorStyle,
    ]);
    useImperativeHandle(fwdRef, () => api);
    const style = useMemo(() => ({
        position: 'relative',
        width: '100%',
        height: '100%',
        overflow: 'hidden',
        touchAction: 'none',
        userSelect: 'none',
        WebkitTapHighlightColor: 'rgba(0, 0, 0, 0)',
    }), []);
    return (jsx(ViewContext.Provider, Object.assign({ value: api }, { children: jsx("div", Object.assign({ style: style, ref: containerRef }, { children: jsx(Renderer, Object.assign({}, rendererProps, { ref: rendererRef }, { children: props.children })) })) })));
});

export { ParentedView as default };
