import vtkGestureCameraManipulator from '@kitware/vtk.js/Interaction/Manipulators/GestureCameraManipulator';
import vtkMouseBoxSelectorManipulator from '@kitware/vtk.js/Interaction/Manipulators/MouseBoxSelectorManipulator';
import vtkMouseCameraTrackballMultiRotateManipulator from '@kitware/vtk.js/Interaction/Manipulators/MouseCameraTrackballMultiRotateManipulator';
import vtkMouseCameraTrackballPanManipulator from '@kitware/vtk.js/Interaction/Manipulators/MouseCameraTrackballPanManipulator';
import vtkMouseCameraTrackballRollManipulator from '@kitware/vtk.js/Interaction/Manipulators/MouseCameraTrackballRollManipulator';
import vtkMouseCameraTrackballRotateManipulator from '@kitware/vtk.js/Interaction/Manipulators/MouseCameraTrackballRotateManipulator';
import vtkMouseCameraTrackballZoomManipulator from '@kitware/vtk.js/Interaction/Manipulators/MouseCameraTrackballZoomManipulator';
import vtkMouseCameraTrackballZoomToMouseManipulator from '@kitware/vtk.js/Interaction/Manipulators/MouseCameraTrackballZoomToMouseManipulator';
import vtkInteractorStyleManipulator from '@kitware/vtk.js/Interaction/Style/InteractorStyleManipulator';
import deepEqual from 'deep-equal';
import { useRef, useCallback, useEffect } from 'react';
import deletionRegistry from '../../utils/DeletionRegistry.js';
import useComparableEffect from '../../utils/useComparableEffect.js';
import useGetterRef from '../../utils/useGetterRef.js';

const settingToManipulator = (setting) => {
    switch (setting.action) {
        case 'Pan':
            return vtkMouseCameraTrackballPanManipulator.newInstance(setting);
        case 'Zoom':
            return vtkMouseCameraTrackballZoomManipulator.newInstance(setting);
        case 'Roll':
            return vtkMouseCameraTrackballRollManipulator.newInstance(setting);
        case 'Rotate':
            return vtkMouseCameraTrackballRotateManipulator.newInstance(setting);
        case 'MultiRotate':
            return vtkMouseCameraTrackballMultiRotateManipulator.newInstance(setting);
        case 'ZoomToMouse':
            return vtkMouseCameraTrackballZoomToMouseManipulator.newInstance(setting);
        case 'Select':
            return vtkMouseBoxSelectorManipulator.newInstance(setting);
    }
};
function useInteractorStyleManipulatorSettings(getStyle, settings) {
    useComparableEffect(() => {
        if (!getStyle().isA('vtkInteractorStyleManipulator'))
            return;
        const style = getStyle();
        style.removeAllManipulators();
        // always add gestures
        style.addGestureManipulator(vtkGestureCameraManipulator.newInstance());
        settings.forEach((setting) => {
            const manip = settingToManipulator(setting);
            // TODO on box select change
            style.addMouseManipulator(manip);
        });
    }, [settings], ([cur], [prev]) => deepEqual(cur, prev));
}
function useInteractorStyle(getInteractor) {
    /**
     * internalStyle: self-managed InteractorStyleManipulator
     * externalStyle: user-set style
     *   - The external style takes precedence over the internal style in getStyle().
     *
     * In a parented view, the style gets set on the interactor whenever
     *   the user hovers over the view.
     * In a single view, the style gets set unconditionally.
     *
     * If the external style is updated:
     *   Single view: set it on the interactor unconditionally
     *   Parented view: set it on the interactor if the previous style
     *     is currently registered to the interactor.
     */
    const [internalStyleRef, getInternalStyle] = useGetterRef(() => {
        const style = vtkInteractorStyleManipulator.newInstance();
        deletionRegistry.register(style, () => style.delete());
        return style;
    });
    const externalStyleRef = useRef(null);
    const getStyle = useCallback(() => {
        var _a;
        return (_a = externalStyleRef.current) !== null && _a !== void 0 ? _a : getInternalStyle();
    }, [getInternalStyle]);
    const setStyle = useCallback((style) => {
        const currentStyle = getStyle();
        const interactor = getInteractor();
        if (!interactor)
            return;
        // Single view: always true
        // Parented view: true if we've hovered over the View
        const isCurrentlySet = interactor.getInteractorStyle() === currentStyle;
        if (isCurrentlySet) {
            interactor.setInteractorStyle(style);
        }
        externalStyleRef.current = style;
    }, [getInteractor, getStyle]);
    useEffect(() => {
        const interactor = getInteractor();
        if (!interactor)
            return;
        deletionRegistry.incRefCount(interactor);
        return () => {
            if (interactor.getInteractorStyle() === getStyle()) {
                interactor.setInteractorStyle(null);
            }
            deletionRegistry.decRefCount(interactor);
            deletionRegistry.markForDeletion(internalStyleRef.current);
            internalStyleRef.current = null;
        };
    }, [getInteractor, getStyle, internalStyleRef]);
    return [getStyle, setStyle];
}

export { useInteractorStyle, useInteractorStyleManipulatorSettings };
