import { useEffect } from 'react';
import { compareShallowObject } from '../../utils/comparators.js';
import deletionRegistry from '../../utils/DeletionRegistry.js';
import useComparableEffect from '../../utils/useComparableEffect.js';
import useGetterRef from '../../utils/useGetterRef.js';
import useUnmount from '../../utils/useUnmount.js';
import { useRendererContext } from '../contexts.js';

/**
 * Returns a vtkProp getter.
 *
 * Must be used in an ordered unmount context for
 * proper unmount order.
 */
function useProp({ constructor, id, props, trackModified }) {
    const renderer = useRendererContext();
    const [actorRef, getActor] = useGetterRef(() => {
        const a = constructor();
        deletionRegistry.register(a, () => a.delete());
        return a;
    });
    useEffect(() => {
        getActor().set({ representationId: id }, true /* noWarning */);
    }, [id, getActor]);
    // add to renderer
    useEffect(() => {
        const actor = getActor();
        const ren = renderer.get();
        deletionRegistry.incRefCount(ren);
        ren.addActor(actor);
        return () => {
            ren.removeActor(actor);
            deletionRegistry.decRefCount(ren);
        };
    }, [renderer, getActor]);
    // set actor props
    useComparableEffect(() => {
        if (!props)
            return;
        trackModified(getActor().set(props));
    }, [props], ([cur], [prev]) => compareShallowObject(cur, prev));
    // cleanup on unmount
    useUnmount(() => {
        if (actorRef.current) {
            deletionRegistry.markForDeletion(actorRef.current);
            actorRef.current = null;
        }
    });
    return getActor;
}

export { useProp as default };
