/**
 * Tracks the usage of a particular object, triggering a deletion callback when refs hit zero and the object is marked for deletion.
 *
 * register() is the entrypoint for tracking an object.
 * markForDeletion() is the counterpart to register(), indicating that the object should be deleted in the future.
 * incRefCount() and decRefCount() should be used to track when code uses/stops using the object.
 * unregister() is used only if there's not reason to continue tracking the object.
 */
class DeletionRegistry {
    constructor() {
        this.refCount = new Map();
        this.callbacks = new Map();
        this.toDelete = new Set();
    }
    get liveCount() {
        return this.refCount.size;
    }
    register(obj, deleteCallback) {
        this.refCount.set(obj, 0);
        this.callbacks.set(obj, deleteCallback);
    }
    unregister(obj) {
        this.refCount.delete(obj);
        this.callbacks.delete(obj);
        this.toDelete.delete(obj);
    }
    markForDeletion(obj) {
        if (this.refCount.has(obj) && this.callbacks.has(obj)) {
            this.toDelete.add(obj);
            this.tryToDispose(obj);
        }
    }
    incRefCount(obj) {
        var _a;
        const count = (_a = this.refCount.get(obj)) !== null && _a !== void 0 ? _a : 0;
        this.refCount.set(obj, count + 1);
    }
    decRefCount(obj) {
        let count = this.refCount.get(obj);
        if (count != null) {
            if (count === 0)
                throw new Error('Cannot decrease ref count below 0');
            count -= 1;
            this.refCount.set(obj, count);
            if (count === 0) {
                this.tryToDispose(obj);
            }
        }
    }
    tryToDispose(obj) {
        var _a;
        if (!this.toDelete.has(obj))
            return;
        const count = (_a = this.refCount.get(obj)) !== null && _a !== void 0 ? _a : Infinity;
        const callback = this.callbacks.get(obj);
        if (count === 0 && callback) {
            callback(obj);
            this.refCount.delete(obj);
            this.callbacks.delete(obj);
            this.toDelete.delete(obj);
        }
    }
}
const deletionRegistry = new DeletionRegistry();

export { DeletionRegistry, deletionRegistry as default };
